<?php

declare(strict_types=1);

namespace Mm\Fieldtrials\Domain\Model;


/**
 * This file is part of the "Field Trials DB" Extension for TYPO3 CMS.
 *
 * For the full copyright and license information, please read the
 * LICENSE.txt file that was distributed with this source code.
 *
 * (c) 2023 Andreas Jud <ajud@millemedia.de>, Millemedia GmbH
 */

/**
 * FieldTrial
 */
class FieldTrial extends \TYPO3\CMS\Extbase\DomainObject\AbstractEntity
{

    /**
     * headline
     *
     * @var string
     */
    protected $headline = null;

    /**
     * subline
     *
     * @var string
     */
    protected $subline = null;

    /**
     * datePlanted
     *
     * @var \DateTime
     */
    protected $datePlanted = null;

    /**
     * locationName
     *
     * @var string
     */
    protected $locationName = null;

    /**
     * farmInformation
     *
     * @var string
     */
    protected $farmInformation = null;

    /**
     * latitude
     *
     * @var string
     */
    protected $latitude = null;

    /**
     * longitude
     *
     * @var string
     */
    protected $longitude = null;

    /**
     * culture
     *
     * @var string
     */
    protected $culture = null;

    /**
     * variety
     *
     * @var string
     */
    protected $variety = null;

    /**
     * firstApplication
     *
     * @var \DateTime
     */
    protected $firstApplication = null;

    /**
     * soilType
     *
     * @var string
     */
    protected $soilType = null;

    /**
     * fertilizationLevel
     *
     * @var string
     */
    protected $fertilizationLevel = null;

    /**
     * description
     *
     * @var string
     */
    protected $description = null;

    /**
     * bbchApplied
     *
     * @var string
     */
    protected $bbchApplied = null;

    /**
     * bbchTargetOrg
     *
     * @var string
     */
    protected $bbchTargetOrg = null;

    /**
     * media
     *
     * @var \TYPO3\CMS\Extbase\Domain\Model\FileReference
     * @TYPO3\CMS\Extbase\Annotation\ORM\Cascade("remove")
     */
    protected $media = null;

    /**
     * addMedia
     *
     * @var \TYPO3\CMS\Extbase\Domain\Model\FileReference
     * @TYPO3\CMS\Extbase\Annotation\ORM\Cascade("remove")
     */
    protected $addMedia = null;

    /**
     * file
     *
     * @var \TYPO3\CMS\Extbase\Domain\Model\FileReference
     * @TYPO3\CMS\Extbase\Annotation\ORM\Cascade("remove")
     */
    protected $file = null;

    /**
     * trialCooperator
     *
     * @var string
     */
    protected $trialCooperator = null;

    /**
     * waterVolume
     *
     * @var string
     */
    protected $waterVolume = null;

    /**
     * plantingMachinery
     *
     * @var \TYPO3\CMS\Extbase\Domain\Model\FileReference
     * @TYPO3\CMS\Extbase\Annotation\ORM\Cascade("remove")
     */
    protected $plantingMachinery = null;

    /**
     * result
     *
     * @var \TYPO3\CMS\Extbase\Persistence\ObjectStorage<\Mm\Fieldtrials\Domain\Model\KeyTakeaways>
     * @TYPO3\CMS\Extbase\Annotation\ORM\Cascade("remove")
     * @TYPO3\CMS\Extbase\Annotation\ORM\Lazy
     */
    protected $result = null;

    /**
     * treatment
     *
     * @var \TYPO3\CMS\Extbase\Persistence\ObjectStorage<\Mm\Fieldtrials\Domain\Model\Treatment>
     * @TYPO3\CMS\Extbase\Annotation\ORM\Cascade("remove")
     * @TYPO3\CMS\Extbase\Annotation\ORM\Lazy
     */
    protected $treatment = null;

    /**
     * territory
     *
     * @var \SJBR\StaticInfoTables\Domain\Model\Territory
     * @TYPO3\CMS\Extbase\Annotation\ORM\Lazy
     */
    protected $territory = null;

    /**
     * county
     *
     * @var \SJBR\StaticInfoTables\Domain\Model\Country
     * @TYPO3\CMS\Extbase\Annotation\ORM\Lazy
     */
    protected $county = null;

    /**
     * state
     *
     * @var \SJBR\StaticInfoTables\Domain\Model\CountryZone
     * @TYPO3\CMS\Extbase\Annotation\ORM\Lazy
     */
    protected $state = null;

    /**
     * __construct
     */
    public function __construct()
    {

        // Do not remove the next line: It would break the functionality
        $this->initializeObject();
    }

    /**
     * Initializes all ObjectStorage properties when model is reconstructed from DB (where __construct is not called)
     * Do not modify this method!
     * It will be rewritten on each save in the extension builder
     * You may modify the constructor of this class instead
     *
     * @return void
     */
    public function initializeObject()
    {
        $this->result = $this->result ?: new \TYPO3\CMS\Extbase\Persistence\ObjectStorage();
        $this->treatment = $this->treatment ?: new \TYPO3\CMS\Extbase\Persistence\ObjectStorage();
    }

    /**
     * Returns the headline
     *
     * @return string
     */
    public function getHeadline()
    {
        return $this->headline;
    }

    /**
     * Sets the headline
     *
     * @param string $headline
     * @return void
     */
    public function setHeadline(string $headline)
    {
        $this->headline = $headline;
    }

    /**
     * Returns the subline
     *
     * @return string
     */
    public function getSubline()
    {
        return $this->subline;
    }

    /**
     * Sets the subline
     *
     * @param string $subline
     * @return void
     */
    public function setSubline(string $subline)
    {
        $this->subline = $subline;
    }

    /**
     * Returns the datePlanted
     *
     * @return \DateTime
     */
    public function getDatePlanted()
    {
        return $this->datePlanted;
    }

    /**
     * Sets the datePlanted
     *
     * @param \DateTime $datePlanted
     * @return void
     */
    public function setDatePlanted(\DateTime $datePlanted)
    {
        $this->datePlanted = $datePlanted;
    }

    /**
     * Returns the locationName
     *
     * @return string
     */
    public function getLocationName()
    {
        return $this->locationName;
    }

    /**
     * Sets the locationName
     *
     * @param string $locationName
     * @return void
     */
    public function setLocationName(string $locationName)
    {
        $this->locationName = $locationName;
    }

    /**
     * Returns the farmInformation
     *
     * @return string
     */
    public function getFarmInformation()
    {
        return $this->farmInformation;
    }

    /**
     * Sets the farmInformation
     *
     * @param string $farmInformation
     * @return void
     */
    public function setFarmInformation(string $farmInformation)
    {
        $this->farmInformation = $farmInformation;
    }

    /**
     * Returns the latitude
     *
     * @return string
     */
    public function getLatitude()
    {
        return $this->latitude;
    }

    /**
     * Sets the latitude
     *
     * @param string $latitude
     * @return void
     */
    public function setLatitude(string $latitude)
    {
        $this->latitude = $latitude;
    }

    /**
     * Returns the longitude
     *
     * @return string
     */
    public function getLongitude()
    {
        return $this->longitude;
    }

    /**
     * Sets the longitude
     *
     * @param string $longitude
     * @return void
     */
    public function setLongitude(string $longitude)
    {
        $this->longitude = $longitude;
    }

    /**
     * Returns the culture
     *
     * @return string
     */
    public function getCulture()
    {
        return $this->culture;
    }

    /**
     * Sets the culture
     *
     * @param string $culture
     * @return void
     */
    public function setCulture(string $culture)
    {
        $this->culture = $culture;
    }

    /**
     * Returns the variety
     *
     * @return string
     */
    public function getVariety()
    {
        return $this->variety;
    }

    /**
     * Sets the variety
     *
     * @param string $variety
     * @return void
     */
    public function setVariety(string $variety)
    {
        $this->variety = $variety;
    }

    /**
     * Returns the firstApplication
     *
     * @return \DateTime
     */
    public function getFirstApplication()
    {
        return $this->firstApplication;
    }

    /**
     * Sets the firstApplication
     *
     * @param \DateTime $firstApplication
     * @return void
     */
    public function setFirstApplication(\DateTime $firstApplication)
    {
        $this->firstApplication = $firstApplication;
    }

    /**
     * Returns the soilType
     *
     * @return string
     */
    public function getSoilType()
    {
        return $this->soilType;
    }

    /**
     * Sets the soilType
     *
     * @param string $soilType
     * @return void
     */
    public function setSoilType(string $soilType)
    {
        $this->soilType = $soilType;
    }

    /**
     * Returns the fertilizationLevel
     *
     * @return string
     */
    public function getFertilizationLevel()
    {
        return $this->fertilizationLevel;
    }

    /**
     * Sets the fertilizationLevel
     *
     * @param string $fertilizationLevel
     * @return void
     */
    public function setFertilizationLevel(string $fertilizationLevel)
    {
        $this->fertilizationLevel = $fertilizationLevel;
    }

    /**
     * Returns the description
     *
     * @return string
     */
    public function getDescription()
    {
        return $this->description;
    }

    /**
     * Sets the description
     *
     * @param string $description
     * @return void
     */
    public function setDescription(string $description)
    {
        $this->description = $description;
    }

    /**
     * Returns the bbchApplied
     *
     * @return string
     */
    public function getBbchApplied()
    {
        return $this->bbchApplied;
    }

    /**
     * Sets the bbchApplied
     *
     * @param string $bbchApplied
     * @return void
     */
    public function setBbchApplied(string $bbchApplied)
    {
        $this->bbchApplied = $bbchApplied;
    }

    /**
     * Returns the bbchTargetOrg
     *
     * @return string
     */
    public function getBbchTargetOrg()
    {
        return $this->bbchTargetOrg;
    }

    /**
     * Sets the bbchTargetOrg
     *
     * @param string $bbchTargetOrg
     * @return void
     */
    public function setBbchTargetOrg(string $bbchTargetOrg)
    {
        $this->bbchTargetOrg = $bbchTargetOrg;
    }

    /**
     * Returns the treatment
     *
     * @return string
     */
    public function getTreatment()
    {
        return $this->treatment;
    }

    /**
     * Sets the treatment
     *
     * @param string $treatment
     * @return void
     */
    public function setTreatment(string $treatment)
    {
        $this->treatment = $treatment;
    }

    /**
     * Returns the media
     *
     * @return \TYPO3\CMS\Extbase\Domain\Model\FileReference
     */
    public function getMedia()
    {
        return $this->media;
    }

    /**
     * Sets the media
     *
     * @param \TYPO3\CMS\Extbase\Domain\Model\FileReference $media
     * @return void
     */
    public function setMedia(\TYPO3\CMS\Extbase\Domain\Model\FileReference $media)
    {
        $this->media = $media;
    }

    /**
     * Returns the addMedia
     *
     * @return \TYPO3\CMS\Extbase\Domain\Model\FileReference
     */
    public function getAddMedia()
    {
        return $this->addMedia;
    }

    /**
     * Sets the addMedia
     *
     * @param \TYPO3\CMS\Extbase\Domain\Model\FileReference $addMedia
     * @return void
     */
    public function setAddMedia(\TYPO3\CMS\Extbase\Domain\Model\FileReference $addMedia)
    {
        $this->addMedia = $addMedia;
    }

    /**
     * Returns the file
     *
     * @return \TYPO3\CMS\Extbase\Domain\Model\FileReference
     */
    public function getFile()
    {
        return $this->file;
    }

    /**
     * Sets the file
     *
     * @param \TYPO3\CMS\Extbase\Domain\Model\FileReference $file
     * @return void
     */
    public function setFile(\TYPO3\CMS\Extbase\Domain\Model\FileReference $file)
    {
        $this->file = $file;
    }

    /**
     * Returns the trialCooperator
     *
     * @return string
     */
    public function getTrialCooperator()
    {
        return $this->trialCooperator;
    }

    /**
     * Sets the trialCooperator
     *
     * @param string $trialCooperator
     * @return void
     */
    public function setTrialCooperator(string $trialCooperator)
    {
        $this->trialCooperator = $trialCooperator;
    }

    /**
     * Returns the waterVolume
     *
     * @return string
     */
    public function getWaterVolume()
    {
        return $this->waterVolume;
    }

    /**
     * Sets the waterVolume
     *
     * @param string $waterVolume
     * @return void
     */
    public function setWaterVolume(string $waterVolume)
    {
        $this->waterVolume = $waterVolume;
    }

    /**
     * Adds a Result
     *
     * @param \Mm\Fieldtrials\Domain\Model\Result $result
     * @return void
     */
    public function addResult(\Mm\Fieldtrials\Domain\Model\Result $result)
    {
        $this->result->attach($result);
    }

    /**
     * Removes a Result
     *
     * @param \Mm\Fieldtrials\Domain\Model\Result $resultToRemove The Result to be removed
     * @return void
     */
    public function removeResult(\Mm\Fieldtrials\Domain\Model\Result $resultToRemove)
    {
        $this->result->detach($resultToRemove);
    }

    /**
     * Returns the result
     *
     * @return \TYPO3\CMS\Extbase\Persistence\ObjectStorage<\Mm\Fieldtrials\Domain\Model\Result>
     */
    public function getResult()
    {
        return $this->result;
    }

    /**
     * Sets the result
     *
     * @param \TYPO3\CMS\Extbase\Persistence\ObjectStorage<\Mm\Fieldtrials\Domain\Model\Result> $result
     * @return void
     */
    public function setResult(\TYPO3\CMS\Extbase\Persistence\ObjectStorage $result)
    {
        $this->result = $result;
    }

    /**
     * Returns the territory
     *
     * @return \SJBR\StaticInfoTables\Domain\Model\Territory
     */
    public function getTerritory()
    {
        return $this->territory;
    }

    /**
     * Sets the territory
     *
     * @param \SJBR\StaticInfoTables\Domain\Model\Territory $territory
     * @return void
     */
    public function setTerritory(\SJBR\StaticInfoTables\Domain\Model\Territory $territory)
    {
        $this->territory = $territory;
    }

    /**
     * Returns the county
     *
     * @return \SJBR\StaticInfoTables\Domain\Model\Country
     */
    public function getCounty()
    {
        return $this->county;
    }

    /**
     * Sets the county
     *
     * @param \SJBR\StaticInfoTables\Domain\Model\Country $county
     * @return void
     */
    public function setCounty(\SJBR\StaticInfoTables\Domain\Model\Country $county)
    {
        $this->county = $county;
    }

    /**
     * Returns the state
     *
     * @return \SJBR\StaticInfoTables\Domain\Model\CountryZone
     */
    public function getState()
    {
        return $this->state;
    }

    /**
     * Sets the state
     *
     * @param \SJBR\StaticInfoTables\Domain\Model\CountryZone $state
     * @return void
     */
    public function setState(\SJBR\StaticInfoTables\Domain\Model\CountryZone $state)
    {
        $this->state = $state;
    }

    /**
     * Returns the plantingMachinery
     *
     * @return \TYPO3\CMS\Extbase\Domain\Model\FileReference
     */
    public function getPlantingMachinery()
    {
        return $this->plantingMachinery;
    }

    /**
     * Sets the plantingMachinery
     *
     * @param \TYPO3\CMS\Extbase\Domain\Model\FileReference $plantingMachinery
     * @return void
     */
    public function setPlantingMachinery(\TYPO3\CMS\Extbase\Domain\Model\FileReference $plantingMachinery)
    {
        $this->plantingMachinery = $plantingMachinery;
    }

    /**
     * Adds a CountryZone
     *
     * @param \SJBR\StaticInfoTables\Domain\Model\CountryZone $treatment
     * @return void
     */
    public function addTreatment(\SJBR\StaticInfoTables\Domain\Model\CountryZone $treatment)
    {
        $this->treatment->attach($treatment);
    }

    /**
     * Removes a CountryZone
     *
     * @param \SJBR\StaticInfoTables\Domain\Model\CountryZone $treatmentToRemove The CountryZone to be removed
     * @return void
     */
    public function removeTreatment(\SJBR\StaticInfoTables\Domain\Model\CountryZone $treatmentToRemove)
    {
        $this->treatment->detach($treatmentToRemove);
    }
}
