<?php

declare(strict_types=1);

namespace Mm\Fieldtrials\Domain\Model;


/**
 * This file is part of the "Field Trials DB" Extension for TYPO3 CMS.
 *
 * For the full copyright and license information, please read the
 * LICENSE.txt file that was distributed with this source code.
 *
 * (c) 2023 Andreas Jud <ajud@millemedia.de>, Millemedia GmbH
 */

/**
 * Product
 */
class Product extends \TYPO3\CMS\Extbase\DomainObject\AbstractEntity
{

    /**
     * name
     *
     * @var string
     */
    protected $name = null;

    /**
     * filedtrial
     *
     * @var \TYPO3\CMS\Extbase\Persistence\ObjectStorage<\Mm\Fieldtrials\Domain\Model\FieldTrial>
     * @TYPO3\CMS\Extbase\Annotation\ORM\Cascade("remove")
     * @TYPO3\CMS\Extbase\Annotation\ORM\Lazy
     */
    protected $filedtrial = null;

    /**
     * productGroup
     *
     * @var \Mm\Fieldtrials\Domain\Model\ProductGroup
     * @TYPO3\CMS\Extbase\Annotation\ORM\Lazy
     */
    protected $productGroup = null;

    /**
     * __construct
     */
    public function __construct()
    {

        // Do not remove the next line: It would break the functionality
        $this->initializeObject();
    }

    /**
     * Initializes all ObjectStorage properties when model is reconstructed from DB (where __construct is not called)
     * Do not modify this method!
     * It will be rewritten on each save in the extension builder
     * You may modify the constructor of this class instead
     *
     * @return void
     */
    public function initializeObject()
    {
        $this->filedtrial = $this->filedtrial ?: new \TYPO3\CMS\Extbase\Persistence\ObjectStorage();
    }

    /**
     * Returns the name
     *
     * @return string
     */
    public function getName()
    {
        return $this->name;
    }

    /**
     * Sets the name
     *
     * @param string $name
     * @return void
     */
    public function setName(string $name)
    {
        $this->name = $name;
    }

    /**
     * Adds a FieldTrial
     *
     * @param \Mm\Fieldtrials\Domain\Model\FieldTrial $filedtrial
     * @return void
     */
    public function addFiledtrial(\Mm\Fieldtrials\Domain\Model\FieldTrial $filedtrial)
    {
        $this->filedtrial->attach($filedtrial);
    }

    /**
     * Removes a FieldTrial
     *
     * @param \Mm\Fieldtrials\Domain\Model\FieldTrial $filedtrialToRemove The FieldTrial to be removed
     * @return void
     */
    public function removeFiledtrial(\Mm\Fieldtrials\Domain\Model\FieldTrial $filedtrialToRemove)
    {
        $this->filedtrial->detach($filedtrialToRemove);
    }

    /**
     * Returns the filedtrial
     *
     * @return \TYPO3\CMS\Extbase\Persistence\ObjectStorage<\Mm\Fieldtrials\Domain\Model\FieldTrial>
     */
    public function getFiledtrial()
    {
        return $this->filedtrial;
    }

    /**
     * Sets the filedtrial
     *
     * @param \TYPO3\CMS\Extbase\Persistence\ObjectStorage<\Mm\Fieldtrials\Domain\Model\FieldTrial> $filedtrial
     * @return void
     */
    public function setFiledtrial(\TYPO3\CMS\Extbase\Persistence\ObjectStorage $filedtrial)
    {
        $this->filedtrial = $filedtrial;
    }

    /**
     * Returns the productGroup
     *
     * @return \Mm\Fieldtrials\Domain\Model\ProductGroup
     */
    public function getProductGroup()
    {
        return $this->productGroup;
    }

    /**
     * Sets the productGroup
     *
     * @param \Mm\Fieldtrials\Domain\Model\ProductGroup $productGroup
     * @return void
     */
    public function setProductGroup(\Mm\Fieldtrials\Domain\Model\ProductGroup $productGroup)
    {
        $this->productGroup = $productGroup;
    }
}
