<?php

declare(strict_types=1);

namespace Mm\Fieldtrials\Tests\Unit\Controller;

use PHPUnit\Framework\MockObject\MockObject;
use TYPO3\TestingFramework\Core\AccessibleObjectInterface;
use TYPO3\TestingFramework\Core\Unit\UnitTestCase;
use TYPO3Fluid\Fluid\View\ViewInterface;

/**
 * Test case
 *
 * @author Andreas Jud <ajud@millemedia.de>
 */
class FieldTrialControllerTest extends UnitTestCase
{
    /**
     * @var \Mm\Fieldtrials\Controller\FieldTrialController|MockObject|AccessibleObjectInterface
     */
    protected $subject;

    protected function setUp(): void
    {
        parent::setUp();
        $this->subject = $this->getMockBuilder($this->buildAccessibleProxy(\Mm\Fieldtrials\Controller\FieldTrialController::class))
            ->onlyMethods(['redirect', 'forward', 'addFlashMessage'])
            ->disableOriginalConstructor()
            ->getMock();
    }

    protected function tearDown(): void
    {
        parent::tearDown();
    }

    /**
     * @test
     */
    public function listActionFetchesAllFieldTrialsFromRepositoryAndAssignsThemToView(): void
    {
        $allFieldTrials = $this->getMockBuilder(\TYPO3\CMS\Extbase\Persistence\ObjectStorage::class)
            ->disableOriginalConstructor()
            ->getMock();

        $fieldTrialRepository = $this->getMockBuilder(\Mm\Fieldtrials\Domain\Repository\FieldTrialRepository::class)
            ->onlyMethods(['findAll'])
            ->disableOriginalConstructor()
            ->getMock();
        $fieldTrialRepository->expects(self::once())->method('findAll')->will(self::returnValue($allFieldTrials));
        $this->subject->_set('fieldTrialRepository', $fieldTrialRepository);

        $view = $this->getMockBuilder(ViewInterface::class)->getMock();
        $view->expects(self::once())->method('assign')->with('fieldTrials', $allFieldTrials);
        $this->subject->_set('view', $view);

        $this->subject->listAction();
    }
}
