<?php

declare(strict_types=1);

namespace Mm\Fieldtrials\Tests\Unit\Domain\Model;

use PHPUnit\Framework\MockObject\MockObject;
use TYPO3\TestingFramework\Core\AccessibleObjectInterface;
use TYPO3\TestingFramework\Core\Unit\UnitTestCase;

/**
 * Test case
 *
 * @author Andreas Jud <ajud@millemedia.de>
 */
class FieldTrialTest extends UnitTestCase
{
    /**
     * @var \Mm\Fieldtrials\Domain\Model\FieldTrial|MockObject|AccessibleObjectInterface
     */
    protected $subject;

    protected function setUp(): void
    {
        parent::setUp();

        $this->subject = $this->getAccessibleMock(
            \Mm\Fieldtrials\Domain\Model\FieldTrial::class,
            ['dummy']
        );
    }

    protected function tearDown(): void
    {
        parent::tearDown();
    }

    /**
     * @test
     */
    public function getHeadlineReturnsInitialValueForString(): void
    {
        self::assertSame(
            '',
            $this->subject->getHeadline()
        );
    }

    /**
     * @test
     */
    public function setHeadlineForStringSetsHeadline(): void
    {
        $this->subject->setHeadline('Conceived at T3CON10');

        self::assertEquals('Conceived at T3CON10', $this->subject->_get('headline'));
    }

    /**
     * @test
     */
    public function getSublineReturnsInitialValueForString(): void
    {
        self::assertSame(
            '',
            $this->subject->getSubline()
        );
    }

    /**
     * @test
     */
    public function setSublineForStringSetsSubline(): void
    {
        $this->subject->setSubline('Conceived at T3CON10');

        self::assertEquals('Conceived at T3CON10', $this->subject->_get('subline'));
    }

    /**
     * @test
     */
    public function getDatePlantedReturnsInitialValueForDateTime(): void
    {
        self::assertEquals(
            null,
            $this->subject->getDatePlanted()
        );
    }

    /**
     * @test
     */
    public function setDatePlantedForDateTimeSetsDatePlanted(): void
    {
        $dateTimeFixture = new \DateTime();
        $this->subject->setDatePlanted($dateTimeFixture);

        self::assertEquals($dateTimeFixture, $this->subject->_get('datePlanted'));
    }

    /**
     * @test
     */
    public function getLocationNameReturnsInitialValueForString(): void
    {
        self::assertSame(
            '',
            $this->subject->getLocationName()
        );
    }

    /**
     * @test
     */
    public function setLocationNameForStringSetsLocationName(): void
    {
        $this->subject->setLocationName('Conceived at T3CON10');

        self::assertEquals('Conceived at T3CON10', $this->subject->_get('locationName'));
    }

    /**
     * @test
     */
    public function getFarmInformationReturnsInitialValueForString(): void
    {
        self::assertSame(
            '',
            $this->subject->getFarmInformation()
        );
    }

    /**
     * @test
     */
    public function setFarmInformationForStringSetsFarmInformation(): void
    {
        $this->subject->setFarmInformation('Conceived at T3CON10');

        self::assertEquals('Conceived at T3CON10', $this->subject->_get('farmInformation'));
    }

    /**
     * @test
     */
    public function getLatitudeReturnsInitialValueForString(): void
    {
        self::assertSame(
            '',
            $this->subject->getLatitude()
        );
    }

    /**
     * @test
     */
    public function setLatitudeForStringSetsLatitude(): void
    {
        $this->subject->setLatitude('Conceived at T3CON10');

        self::assertEquals('Conceived at T3CON10', $this->subject->_get('latitude'));
    }

    /**
     * @test
     */
    public function getLongitudeReturnsInitialValueForString(): void
    {
        self::assertSame(
            '',
            $this->subject->getLongitude()
        );
    }

    /**
     * @test
     */
    public function setLongitudeForStringSetsLongitude(): void
    {
        $this->subject->setLongitude('Conceived at T3CON10');

        self::assertEquals('Conceived at T3CON10', $this->subject->_get('longitude'));
    }

    /**
     * @test
     */
    public function getCultureReturnsInitialValueForString(): void
    {
        self::assertSame(
            '',
            $this->subject->getCulture()
        );
    }

    /**
     * @test
     */
    public function setCultureForStringSetsCulture(): void
    {
        $this->subject->setCulture('Conceived at T3CON10');

        self::assertEquals('Conceived at T3CON10', $this->subject->_get('culture'));
    }

    /**
     * @test
     */
    public function getVarietyReturnsInitialValueForString(): void
    {
        self::assertSame(
            '',
            $this->subject->getVariety()
        );
    }

    /**
     * @test
     */
    public function setVarietyForStringSetsVariety(): void
    {
        $this->subject->setVariety('Conceived at T3CON10');

        self::assertEquals('Conceived at T3CON10', $this->subject->_get('variety'));
    }

    /**
     * @test
     */
    public function getFirstApplicationReturnsInitialValueForDateTime(): void
    {
        self::assertEquals(
            null,
            $this->subject->getFirstApplication()
        );
    }

    /**
     * @test
     */
    public function setFirstApplicationForDateTimeSetsFirstApplication(): void
    {
        $dateTimeFixture = new \DateTime();
        $this->subject->setFirstApplication($dateTimeFixture);

        self::assertEquals($dateTimeFixture, $this->subject->_get('firstApplication'));
    }

    /**
     * @test
     */
    public function getSoilTypeReturnsInitialValueForString(): void
    {
        self::assertSame(
            '',
            $this->subject->getSoilType()
        );
    }

    /**
     * @test
     */
    public function setSoilTypeForStringSetsSoilType(): void
    {
        $this->subject->setSoilType('Conceived at T3CON10');

        self::assertEquals('Conceived at T3CON10', $this->subject->_get('soilType'));
    }

    /**
     * @test
     */
    public function getFertilizationLevelReturnsInitialValueForString(): void
    {
        self::assertSame(
            '',
            $this->subject->getFertilizationLevel()
        );
    }

    /**
     * @test
     */
    public function setFertilizationLevelForStringSetsFertilizationLevel(): void
    {
        $this->subject->setFertilizationLevel('Conceived at T3CON10');

        self::assertEquals('Conceived at T3CON10', $this->subject->_get('fertilizationLevel'));
    }

    /**
     * @test
     */
    public function getDescriptionReturnsInitialValueForString(): void
    {
        self::assertSame(
            '',
            $this->subject->getDescription()
        );
    }

    /**
     * @test
     */
    public function setDescriptionForStringSetsDescription(): void
    {
        $this->subject->setDescription('Conceived at T3CON10');

        self::assertEquals('Conceived at T3CON10', $this->subject->_get('description'));
    }

    /**
     * @test
     */
    public function getBbchAppliedReturnsInitialValueForString(): void
    {
        self::assertSame(
            '',
            $this->subject->getBbchApplied()
        );
    }

    /**
     * @test
     */
    public function setBbchAppliedForStringSetsBbchApplied(): void
    {
        $this->subject->setBbchApplied('Conceived at T3CON10');

        self::assertEquals('Conceived at T3CON10', $this->subject->_get('bbchApplied'));
    }

    /**
     * @test
     */
    public function getBbchTargetOrgReturnsInitialValueForString(): void
    {
        self::assertSame(
            '',
            $this->subject->getBbchTargetOrg()
        );
    }

    /**
     * @test
     */
    public function setBbchTargetOrgForStringSetsBbchTargetOrg(): void
    {
        $this->subject->setBbchTargetOrg('Conceived at T3CON10');

        self::assertEquals('Conceived at T3CON10', $this->subject->_get('bbchTargetOrg'));
    }

    /**
     * @test
     */
    public function getMediaReturnsInitialValueForFileReference(): void
    {
        self::assertEquals(
            null,
            $this->subject->getMedia()
        );
    }

    /**
     * @test
     */
    public function setMediaForFileReferenceSetsMedia(): void
    {
        $fileReferenceFixture = new \TYPO3\CMS\Extbase\Domain\Model\FileReference();
        $this->subject->setMedia($fileReferenceFixture);

        self::assertEquals($fileReferenceFixture, $this->subject->_get('media'));
    }

    /**
     * @test
     */
    public function getAddMediaReturnsInitialValueForFileReference(): void
    {
        self::assertEquals(
            null,
            $this->subject->getAddMedia()
        );
    }

    /**
     * @test
     */
    public function setAddMediaForFileReferenceSetsAddMedia(): void
    {
        $fileReferenceFixture = new \TYPO3\CMS\Extbase\Domain\Model\FileReference();
        $this->subject->setAddMedia($fileReferenceFixture);

        self::assertEquals($fileReferenceFixture, $this->subject->_get('addMedia'));
    }

    /**
     * @test
     */
    public function getFileReturnsInitialValueForFileReference(): void
    {
        self::assertEquals(
            null,
            $this->subject->getFile()
        );
    }

    /**
     * @test
     */
    public function setFileForFileReferenceSetsFile(): void
    {
        $fileReferenceFixture = new \TYPO3\CMS\Extbase\Domain\Model\FileReference();
        $this->subject->setFile($fileReferenceFixture);

        self::assertEquals($fileReferenceFixture, $this->subject->_get('file'));
    }

    /**
     * @test
     */
    public function getTrialCooperatorReturnsInitialValueForString(): void
    {
        self::assertSame(
            '',
            $this->subject->getTrialCooperator()
        );
    }

    /**
     * @test
     */
    public function setTrialCooperatorForStringSetsTrialCooperator(): void
    {
        $this->subject->setTrialCooperator('Conceived at T3CON10');

        self::assertEquals('Conceived at T3CON10', $this->subject->_get('trialCooperator'));
    }

    /**
     * @test
     */
    public function getWaterVolumeReturnsInitialValueForString(): void
    {
        self::assertSame(
            '',
            $this->subject->getWaterVolume()
        );
    }

    /**
     * @test
     */
    public function setWaterVolumeForStringSetsWaterVolume(): void
    {
        $this->subject->setWaterVolume('Conceived at T3CON10');

        self::assertEquals('Conceived at T3CON10', $this->subject->_get('waterVolume'));
    }

    /**
     * @test
     */
    public function getPlantingMachineryReturnsInitialValueForFileReference(): void
    {
        self::assertEquals(
            null,
            $this->subject->getPlantingMachinery()
        );
    }

    /**
     * @test
     */
    public function setPlantingMachineryForFileReferenceSetsPlantingMachinery(): void
    {
        $fileReferenceFixture = new \TYPO3\CMS\Extbase\Domain\Model\FileReference();
        $this->subject->setPlantingMachinery($fileReferenceFixture);

        self::assertEquals($fileReferenceFixture, $this->subject->_get('plantingMachinery'));
    }

    /**
     * @test
     */
    public function getResultReturnsInitialValueForKeyTakeaways(): void
    {
        $newObjectStorage = new \TYPO3\CMS\Extbase\Persistence\ObjectStorage();
        self::assertEquals(
            $newObjectStorage,
            $this->subject->getResult()
        );
    }

    /**
     * @test
     */
    public function setResultForObjectStorageContainingKeyTakeawaysSetsResult(): void
    {
        $result = new \Mm\Fieldtrials\Domain\Model\KeyTakeaways();
        $objectStorageHoldingExactlyOneResult = new \TYPO3\CMS\Extbase\Persistence\ObjectStorage();
        $objectStorageHoldingExactlyOneResult->attach($result);
        $this->subject->setResult($objectStorageHoldingExactlyOneResult);

        self::assertEquals($objectStorageHoldingExactlyOneResult, $this->subject->_get('result'));
    }

    /**
     * @test
     */
    public function addResultToObjectStorageHoldingResult(): void
    {
        $result = new \Mm\Fieldtrials\Domain\Model\KeyTakeaways();
        $resultObjectStorageMock = $this->getMockBuilder(\TYPO3\CMS\Extbase\Persistence\ObjectStorage::class)
            ->onlyMethods(['attach'])
            ->disableOriginalConstructor()
            ->getMock();

        $resultObjectStorageMock->expects(self::once())->method('attach')->with(self::equalTo($result));
        $this->subject->_set('result', $resultObjectStorageMock);

        $this->subject->addResult($result);
    }

    /**
     * @test
     */
    public function removeResultFromObjectStorageHoldingResult(): void
    {
        $result = new \Mm\Fieldtrials\Domain\Model\KeyTakeaways();
        $resultObjectStorageMock = $this->getMockBuilder(\TYPO3\CMS\Extbase\Persistence\ObjectStorage::class)
            ->onlyMethods(['detach'])
            ->disableOriginalConstructor()
            ->getMock();

        $resultObjectStorageMock->expects(self::once())->method('detach')->with(self::equalTo($result));
        $this->subject->_set('result', $resultObjectStorageMock);

        $this->subject->removeResult($result);
    }

    /**
     * @test
     */
    public function getTreatmentReturnsInitialValueForTreatment(): void
    {
        $newObjectStorage = new \TYPO3\CMS\Extbase\Persistence\ObjectStorage();
        self::assertEquals(
            $newObjectStorage,
            $this->subject->getTreatment()
        );
    }

    /**
     * @test
     */
    public function setTreatmentForObjectStorageContainingTreatmentSetsTreatment(): void
    {
        $treatment = new \Mm\Fieldtrials\Domain\Model\Treatment();
        $objectStorageHoldingExactlyOneTreatment = new \TYPO3\CMS\Extbase\Persistence\ObjectStorage();
        $objectStorageHoldingExactlyOneTreatment->attach($treatment);
        $this->subject->setTreatment($objectStorageHoldingExactlyOneTreatment);

        self::assertEquals($objectStorageHoldingExactlyOneTreatment, $this->subject->_get('treatment'));
    }

    /**
     * @test
     */
    public function addTreatmentToObjectStorageHoldingTreatment(): void
    {
        $treatment = new \Mm\Fieldtrials\Domain\Model\Treatment();
        $treatmentObjectStorageMock = $this->getMockBuilder(\TYPO3\CMS\Extbase\Persistence\ObjectStorage::class)
            ->onlyMethods(['attach'])
            ->disableOriginalConstructor()
            ->getMock();

        $treatmentObjectStorageMock->expects(self::once())->method('attach')->with(self::equalTo($treatment));
        $this->subject->_set('treatment', $treatmentObjectStorageMock);

        $this->subject->addTreatment($treatment);
    }

    /**
     * @test
     */
    public function removeTreatmentFromObjectStorageHoldingTreatment(): void
    {
        $treatment = new \Mm\Fieldtrials\Domain\Model\Treatment();
        $treatmentObjectStorageMock = $this->getMockBuilder(\TYPO3\CMS\Extbase\Persistence\ObjectStorage::class)
            ->onlyMethods(['detach'])
            ->disableOriginalConstructor()
            ->getMock();

        $treatmentObjectStorageMock->expects(self::once())->method('detach')->with(self::equalTo($treatment));
        $this->subject->_set('treatment', $treatmentObjectStorageMock);

        $this->subject->removeTreatment($treatment);
    }

    /**
     * @test
     */
    public function getTerritoryReturnsInitialValueForTerritory(): void
    {
    }

    /**
     * @test
     */
    public function setTerritoryForTerritorySetsTerritory(): void
    {
    }

    /**
     * @test
     */
    public function getCountyReturnsInitialValueForCountry(): void
    {
    }

    /**
     * @test
     */
    public function setCountyForCountrySetsCounty(): void
    {
    }

    /**
     * @test
     */
    public function getStateReturnsInitialValueForCountryZone(): void
    {
    }

    /**
     * @test
     */
    public function setStateForCountryZoneSetsState(): void
    {
    }
}
