<?php

declare(strict_types=1);

namespace Mm\Fieldtrials\Tests\Unit\Domain\Model;

use PHPUnit\Framework\MockObject\MockObject;
use TYPO3\TestingFramework\Core\AccessibleObjectInterface;
use TYPO3\TestingFramework\Core\Unit\UnitTestCase;

/**
 * Test case
 *
 * @author Andreas Jud <ajud@millemedia.de>
 */
class ProductTest extends UnitTestCase
{
    /**
     * @var \Mm\Fieldtrials\Domain\Model\Product|MockObject|AccessibleObjectInterface
     */
    protected $subject;

    protected function setUp(): void
    {
        parent::setUp();

        $this->subject = $this->getAccessibleMock(
            \Mm\Fieldtrials\Domain\Model\Product::class,
            ['dummy']
        );
    }

    protected function tearDown(): void
    {
        parent::tearDown();
    }

    /**
     * @test
     */
    public function getNameReturnsInitialValueForString(): void
    {
        self::assertSame(
            '',
            $this->subject->getName()
        );
    }

    /**
     * @test
     */
    public function setNameForStringSetsName(): void
    {
        $this->subject->setName('Conceived at T3CON10');

        self::assertEquals('Conceived at T3CON10', $this->subject->_get('name'));
    }

    /**
     * @test
     */
    public function getFiledtrialReturnsInitialValueForFieldTrial(): void
    {
        $newObjectStorage = new \TYPO3\CMS\Extbase\Persistence\ObjectStorage();
        self::assertEquals(
            $newObjectStorage,
            $this->subject->getFiledtrial()
        );
    }

    /**
     * @test
     */
    public function setFiledtrialForObjectStorageContainingFieldTrialSetsFiledtrial(): void
    {
        $filedtrial = new \Mm\Fieldtrials\Domain\Model\FieldTrial();
        $objectStorageHoldingExactlyOneFiledtrial = new \TYPO3\CMS\Extbase\Persistence\ObjectStorage();
        $objectStorageHoldingExactlyOneFiledtrial->attach($filedtrial);
        $this->subject->setFiledtrial($objectStorageHoldingExactlyOneFiledtrial);

        self::assertEquals($objectStorageHoldingExactlyOneFiledtrial, $this->subject->_get('filedtrial'));
    }

    /**
     * @test
     */
    public function addFiledtrialToObjectStorageHoldingFiledtrial(): void
    {
        $filedtrial = new \Mm\Fieldtrials\Domain\Model\FieldTrial();
        $filedtrialObjectStorageMock = $this->getMockBuilder(\TYPO3\CMS\Extbase\Persistence\ObjectStorage::class)
            ->onlyMethods(['attach'])
            ->disableOriginalConstructor()
            ->getMock();

        $filedtrialObjectStorageMock->expects(self::once())->method('attach')->with(self::equalTo($filedtrial));
        $this->subject->_set('filedtrial', $filedtrialObjectStorageMock);

        $this->subject->addFiledtrial($filedtrial);
    }

    /**
     * @test
     */
    public function removeFiledtrialFromObjectStorageHoldingFiledtrial(): void
    {
        $filedtrial = new \Mm\Fieldtrials\Domain\Model\FieldTrial();
        $filedtrialObjectStorageMock = $this->getMockBuilder(\TYPO3\CMS\Extbase\Persistence\ObjectStorage::class)
            ->onlyMethods(['detach'])
            ->disableOriginalConstructor()
            ->getMock();

        $filedtrialObjectStorageMock->expects(self::once())->method('detach')->with(self::equalTo($filedtrial));
        $this->subject->_set('filedtrial', $filedtrialObjectStorageMock);

        $this->subject->removeFiledtrial($filedtrial);
    }

    /**
     * @test
     */
    public function getProductGroupReturnsInitialValueForProductGroup(): void
    {
        self::assertEquals(
            null,
            $this->subject->getProductGroup()
        );
    }

    /**
     * @test
     */
    public function setProductGroupForProductGroupSetsProductGroup(): void
    {
        $productGroupFixture = new \Mm\Fieldtrials\Domain\Model\ProductGroup();
        $this->subject->setProductGroup($productGroupFixture);

        self::assertEquals($productGroupFixture, $this->subject->_get('productGroup'));
    }
}
