<?php

declare(strict_types=1);

namespace Mm\Fieldtrials\Tests\Unit\Domain\Model;

use PHPUnit\Framework\MockObject\MockObject;
use TYPO3\TestingFramework\Core\AccessibleObjectInterface;
use TYPO3\TestingFramework\Core\Unit\UnitTestCase;

/**
 * Test case
 *
 * @author Andreas Jud <ajud@millemedia.de>
 */
class ResultTest extends UnitTestCase
{
    /**
     * @var \Mm\Fieldtrials\Domain\Model\Result|MockObject|AccessibleObjectInterface
     */
    protected $subject;

    protected function setUp(): void
    {
        parent::setUp();

        $this->subject = $this->getAccessibleMock(
            \Mm\Fieldtrials\Domain\Model\Result::class,
            ['dummy']
        );
    }

    protected function tearDown(): void
    {
        parent::tearDown();
    }

    /**
     * @test
     */
    public function getHeadlineReturnsInitialValueForString(): void
    {
        self::assertSame(
            '',
            $this->subject->getHeadline()
        );
    }

    /**
     * @test
     */
    public function setHeadlineForStringSetsHeadline(): void
    {
        $this->subject->setHeadline('Conceived at T3CON10');

        self::assertEquals('Conceived at T3CON10', $this->subject->_get('headline'));
    }

    /**
     * @test
     */
    public function getSublineReturnsInitialValueForString(): void
    {
        self::assertSame(
            '',
            $this->subject->getSubline()
        );
    }

    /**
     * @test
     */
    public function setSublineForStringSetsSubline(): void
    {
        $this->subject->setSubline('Conceived at T3CON10');

        self::assertEquals('Conceived at T3CON10', $this->subject->_get('subline'));
    }

    /**
     * @test
     */
    public function getTextReturnsInitialValueForString(): void
    {
        self::assertSame(
            '',
            $this->subject->getText()
        );
    }

    /**
     * @test
     */
    public function setTextForStringSetsText(): void
    {
        $this->subject->setText('Conceived at T3CON10');

        self::assertEquals('Conceived at T3CON10', $this->subject->_get('text'));
    }

    /**
     * @test
     */
    public function getMediaReturnsInitialValueForFileReference(): void
    {
        self::assertEquals(
            null,
            $this->subject->getMedia()
        );
    }

    /**
     * @test
     */
    public function setMediaForFileReferenceSetsMedia(): void
    {
        $fileReferenceFixture = new \TYPO3\CMS\Extbase\Domain\Model\FileReference();
        $this->subject->setMedia($fileReferenceFixture);

        self::assertEquals($fileReferenceFixture, $this->subject->_get('media'));
    }
}
