const diamondsAmount = 14, // Must be even number
    scaleFactor = 75,
    blurFactor = 30;

document.addEventListener('DOMContentLoaded', function()
{
    // Populate_diamonds
    generate_diamonds(diamondsAmount/2,'left');
    generate_diamonds(diamondsAmount/2,'right');

    // Init_animation
    diamond_animation();
    window.addEventListener('scroll', function()
    {
        diamond_animation();
    });
});

// Diamond_animation
function diamond_animation()
{
    document.querySelectorAll('.bg-diamond').forEach(function(diamond, index)
    {
        let svg = diamond.querySelector('svg');

        if( isInViewport(diamond) )
        {
            diamond.classList.add('in-view');
            if( diamond.classList.contains('bg-diamond-left') )
            {
                svg.style.transform = 'scale('+(get_relativeY(diamond) / scaleFactor)+') translateX(-'+get_relativeY(diamond)+'%)';
            } else {
                svg.style.transform = 'scale('+(get_relativeY(diamond) / scaleFactor)+') translateX('+get_relativeY(diamond)+'%)';
            }
            svg.style.filter = 'blur('+(scaleFactor / get_relativeY(diamond))+'px)';
            svg.style.opacity = (get_relativeY(diamond) / 50);
        } else {
            diamond.classList.remove('in-view');
            svg.style.transform = 'scale(0) translateX(0%)';
            svg.style.filter = 'blur(0)';
        }
    });
}
// Diamond_animation END

// Random_int
function random_int( min, max ){
    if( max ) {
        return Math.floor(Math.random() * (max - min + 1)) + min;
    } else {
        return Math.floor( Math.random() * ( min + 1 ) );
    }
}
// Random_int END


// Generate_diamonds
function generate_diamonds(amount,orientation)
{
    for(let i = 0; i < amount; i++)
    {
        let element = createDiamond(i,amount,orientation);
        document.getElementById('page-wrapper').appendChild(element);
    }
}
// Generate_diamonds END

// Get_relative_Y
function get_relativeY(element)
{
    let top = element.offsetTop,
        bottom = element.offsetTop + window.innerHeight,
        currentPos = window.scrollY + window.innerHeight;

    return (bottom - currentPos) / (bottom - top) * 100;
}
// Get_relative_Y END

function isInViewport(element) {
    var rect = element.getBoundingClientRect();
    var html = document.documentElement;
    return (
        rect.top >= 0 &&
        rect.left >= 0 &&
        rect.bottom <= (window.innerHeight || html.clientHeight) &&
        rect.right <= (window.innerWidth || html.clientWidth)
    );
}

// Calc_distance
function calc_distance(a,b)
{
    return (a >= b ) ? a-b : b-a;
}
// Calc_distance END

// Create_diamond
function createDiamond(key,amount,orientation)
{
    let diamond = document.createElement('div'),
        totalHeight = document.getElementById('page-content').scrollHeight,
        startPosition = document.getElementById('page-content').offsetTop,
        sectionHeight = totalHeight/amount,
        top = (startPosition + (sectionHeight * key)),
        bottom = (startPosition + ((sectionHeight * key) + sectionHeight)),
        positionY = random_int(top,bottom);


    // Create_SVG
    diamond.classList.add('bg-diamond', 'bg-diamond-' + orientation);
    diamond.style.width = random_int(30,250)+'px';
    diamond.style.top = positionY + 'px';
    diamond.innerHTML = `
    <svg xmlns="http://www.w3.org/2000/svg" width="176" height="101" viewBox="0 0 176 101" fill="none">
        <g clip-path="url(#diamond_` + key + `)">
          <path d="M87.4672 -3.82618e-06L175 50.5328L87.4672 101L-0.065615 50.5328L87.4672 -3.82618e-06Z" fill="#F6635D"/>
          <path d="M87.4672 -3.82618e-06L175 50.5328L87.4672 101L-0.065615 50.5328L87.4672 -3.82618e-06Z" fill="#F6635D"/>
        </g>
        <defs>
          <clipPath id="diamond_` + key + `">
            <rect width="101" height="175" fill="white" transform="translate(175) rotate(90)"/>
          </clipPath>
        </defs>
    </svg>`;
    // Create_SVG END

    return diamond;
}
// Create_diamond END