/*************************************
 USAGE:
 add a trigger by using the function
 window.scroll_tracker.add_trigger( init_func, trigger_func );
 OR
 <div scroll_function="diamond_video"></div>
 window.diamond_video = function(scrolltracker, element){ ... }

 ****************************************************************************/


window.scroll_tracker = (function(){

	if(window.scroll_tracker) {

		window.scroll_tracker.refresh();

		return(window.scroll_tracker);
	}

	var st = {};
	st.scroll_check_delay = 0;// in millseconds

	st.scroll_elements = [];
	st.initialized = false;//window load didn't start


	st.add_trigger = function( _init, _viewtest ){

		var o = {};

		o._viewtest	= _viewtest;
		o._init		= _init;
		o._parent	= this;

		this.scroll_elements.push(o);

		if( this.initialized && !o._initialized ){
			//WINDOW ON LOAD IS OVER
			o._initialized = true;
			o._init( this );
			this.trigger();
		}
	};
	st.remove_trigger = function( _trigger ){

		var i = this.scroll_elements.length;
		while(i--){

			if( this.scroll_elements[i] === _trigger ){
				//delete this.scroll_elements[i];
				this.scroll_elements.splice(i,1);
				break;
			}
		}
	};

	st.init = function(){

		var a = document.querySelectorAll("[scroll_function]"),
			j = a.length, i, o, s;

		while( j-- ){

			o = a[j];
			i = this.scroll_elements.length;

			while( i-- ){

				if( o === this.scroll_elements[i]._element ){

					break;
				}
			}

			if( i == -1 ){

				s = a[j].getAttribute("scroll_function");

				if(s && window[s]){

					o = {};

					o._element = a[j];

					o._function = window[s];
					o._function(this, o._element);
					o._initialized = true;

					this.scroll_elements.push(o);
				}
			}
		}

		i = this.scroll_elements.length;

		while( i-- ){

			o = this.scroll_elements[i];

			if( !o._initialized ){

				o._initialized = true;
				o._init && o._init( this );
			}
		}

		window.addEventListener('scroll', this.trigger );
		window.addEventListener('resize', this.trigger );

		this.initialized = true;

	}.bind(st);

	st.trigger = function(e) {

		if(this.scroll_check_delay){

			window.removeEventListener('scroll', this.trigger );
			window.removeEventListener('resize', this.trigger );

			if( !isNaN(this.timeout) ){
				window.clearTimeout(this.timeout);
			}

			this.timeout = setTimeout( this.refresh, this.scroll_check_delay, e );
		} else {
			this.refresh(e);
		}

	}.bind(st);


	// call this function to refresh scoll checks
	st.refresh = function(e){

		if( !isNaN(this.timeout) ){
			// put it in this condition in case refresh was fired not by timeout
			delete this.timeout;
			window.addEventListener('scroll', this.trigger );
			window.addEventListener('resize', this.trigger );
		}

		var i = this.scroll_elements.length;

		while(i--){

			var o = this.scroll_elements[i];
			o._viewtest && o._viewtest(this);
			o._function && o._function(this, o._element, e);
		}
	}.bind(st);

	/////////////////////////// UNIVERSAL FUNCTIONS /////////////////////////////

	st.element_in_view = function(e, as_much_as_possible) {

		var rect = e.getBoundingClientRect();

		var center = (rect.top + rect.bottom)*0.5+10;

		//CENTER IS IN SCREEN
		if( !as_much_as_possible ){

			return( center >= 0 && center < (window.innerHeight || document.documentElement.clientHeight) );
		} else {

			if( rect.bottom - rect.top <= (window.innerHeight || document.documentElement.clientHeight) ){
				// FITS IN SCREEN
				return( rect.bottom <= (window.innerHeight || document.documentElement.clientHeight) && rect.top >= 0 );
			} else {
				// DOESN'T FIT
				var center = rect.top + (rect.bottom - rect.top) * 0.5;
				var frame_size = ( window.innerHeight || document.documentElement.clientHeight );

				var border = Math.min( 40, 0.2*frame_size );

				return(center > border && center < frame_size - border*2);
			}
		}
	};

	st.fits_view_y = function(e){

		var rect = e.getBoundingClientRect();
		return( (window.innerHeight || document.documentElement.clientHeight) - (rect.bottom - rect.top) );
	};

	st.is_completely_in_view = function(e, full){

		var rect = e.getBoundingClientRect();

		var pos = {
			"top":		Math.min( rect.top, 0 ),
			"bottom":	Math.max( rect.bottom-(window.innerHeight || document.documentElement.clientHeight), 0 ) };

		if(full){
			pos.top_full = rect.top;
			pos.bottom_full = rect.bottom-(window.innerHeight || document.documentElement.clientHeight);
		}

		return(pos);
	};
	st.completely_out_of_view = function(e){

		var rect = e.getBoundingClientRect();
		return( (window.innerHeight || document.documentElement.clientHeight) -rect.top <= 0 || rect.bottom <= 0 );
	};

	//if the whole container (e) doesn't fit in screen, scroll towards the imporant_child element
	st.scroll_to = function(container, important_child){

		var fits = this.fits_view_y(container);

		if( fits >= 0 || !important_child ){

			var pos = is_completely_in_view(container);
			var border = Math.min( 50, Math.floor( fits * 0.5 ) );

			if( pos.top ){
				window.scrollBy({
					top: pos.top - border,
					left: 0,
					behavior: 'smooth'
				});
			} else if( pos.bottom ){
				window.scrollBy({
					top: pos.bottom + border,
					left: 0,
					behavior: 'smooth'
				});
			}
		} else {

			var fits = this.fits_view_y(important_child);

			var pos = this.is_completely_in_view(important_child);
			var border = Math.min( 50, Math.floor( fits * 0.5 ) );

			if( pos.top && pos.bottom ){

				window.scrollBy({
					top: pos.top*0.5 + pos.bottom*0.5,
					left: 0,
					behavior: 'smooth'
				});
			} else if( pos.top ){
				window.scrollBy({
					top: pos.top - border,
					left: 0,
					behavior: 'smooth'
				});
			} else if( pos.bottom ){
				window.scrollBy({
					top: pos.bottom + border,
					left: 0,
					behavior: 'smooth'
				});
			}
		}
	};

	performance.timing.loadEventEnd && st.init() || window.addEventListener( "load", st.init );

	return(st);
})();





